/*
 * SPDX-FileCopyrightText: NVIDIA CORPORATION & AFFILIATES
 * Copyright (c) 2014-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: LicenseRef-NvidiaProprietary
 *
 * NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
 * property and proprietary rights in and to this material, related
 * documentation and any modifications thereto. Any use, reproduction,
 * disclosure or distribution of this material and related documentation
 * without an express license agreement from NVIDIA CORPORATION or
 * its affiliates is strictly prohibited.
 */

#ifndef NETDEV_DOCA_H
#define NETDEV_DOCA_H

#include <config.h>

#include "netdev-doca-ext.h"
#include "openvswitch/compiler.h"
#include "smap.h"

struct netdev_class;
struct dp_packet;
struct netdev;

#ifdef DPDK_NETDEV

#include <rte_flow.h>

void netdev_doca_register(const struct smap *);

bool netdev_doca_flow_api_supported(struct netdev *);

int
netdev_doca_rte_flow_destroy(struct netdev *netdev,
                             struct rte_flow *rte_flow,
                             struct rte_flow_error *error,
                             bool esw_port_id);
struct rte_flow *
netdev_doca_rte_flow_create(struct netdev *netdev,
                            const struct rte_flow_attr *attr,
                            const struct rte_flow_item *items,
                            const struct rte_flow_action *actions,
                            struct rte_flow_error *error);
int
netdev_doca_rte_flow_query_count(struct netdev *netdev,
                                 struct rte_flow *rte_flow,
                                 struct rte_flow_query_count *query,
                                 struct rte_flow_error *error);
int
netdev_doca_get_port_id(struct netdev *netdev);

static inline void
netdev_doca_set_error(struct rte_flow_error *error, enum rte_flow_error_type type)
{
    if (!error) {
        return;
    }
    error->type = type;
    error->cause = NULL;
    error->message = NULL;
}

#ifdef ALLOW_EXPERIMENTAL_API
#ifdef OVS_UPSTREAM

int netdev_doca_rte_flow_tunnel_decap_set(struct netdev *,
                                          struct rte_flow_tunnel *,
                                          struct rte_flow_action **,
                                          uint32_t *num_of_actions,
                                          struct rte_flow_error *);
int netdev_doca_rte_flow_tunnel_match(struct netdev *,
                                      struct rte_flow_tunnel *,
                                      struct rte_flow_item **,
                                      uint32_t *num_of_items,
                                      struct rte_flow_error *);
int netdev_doca_rte_flow_get_restore_info(struct netdev *,
                                          struct dp_packet *,
                                          struct rte_flow_restore_info *,
                                          struct rte_flow_error *);
int netdev_doca_rte_flow_tunnel_action_decap_release(struct netdev *,
                                                     struct rte_flow_action *,
                                                     uint32_t num_of_actions,
                                                     struct rte_flow_error *);
int netdev_doca_rte_flow_tunnel_item_release(struct netdev *,
                                             struct rte_flow_item *,
                                             uint32_t num_of_items,
                                             struct rte_flow_error *);

#else

static inline int
netdev_doca_rte_flow_tunnel_decap_set(
    struct netdev *netdev OVS_UNUSED,
    struct rte_flow_tunnel *tunnel OVS_UNUSED,
    struct rte_flow_action **actions OVS_UNUSED,
    uint32_t *num_of_actions OVS_UNUSED,
    struct rte_flow_error *error)
{
    netdev_doca_set_error(error, RTE_FLOW_ERROR_TYPE_ACTION);
    return -1;
}

static inline int
netdev_doca_rte_flow_tunnel_match(struct netdev *netdev OVS_UNUSED,
                                  struct rte_flow_tunnel *tunnel OVS_UNUSED,
                                  struct rte_flow_item **items OVS_UNUSED,
                                  uint32_t *num_of_items OVS_UNUSED,
                                  struct rte_flow_error *error)
{
    netdev_doca_set_error(error, RTE_FLOW_ERROR_TYPE_ITEM);
    return -1;
}

static inline int
netdev_doca_rte_flow_get_restore_info(
    struct netdev *netdev OVS_UNUSED,
    struct dp_packet *p OVS_UNUSED,
    struct rte_flow_restore_info *info OVS_UNUSED,
    struct rte_flow_error *error)
{
    netdev_doca_set_error(error, RTE_FLOW_ERROR_TYPE_ATTR);
    return -1;
}

static inline int
netdev_doca_rte_flow_tunnel_action_decap_release(
    struct netdev *netdev OVS_UNUSED,
    struct rte_flow_action *actions OVS_UNUSED,
    uint32_t num_of_actions OVS_UNUSED,
    struct rte_flow_error *error)
{
    netdev_doca_set_error(error, RTE_FLOW_ERROR_TYPE_NONE);
    return 0;
}

static inline int
netdev_doca_rte_flow_tunnel_item_release(
    struct netdev *netdev OVS_UNUSED,
    struct rte_flow_item *items OVS_UNUSED,
    uint32_t num_of_items OVS_UNUSED,
    struct rte_flow_error *error)
{
    netdev_doca_set_error(error, RTE_FLOW_ERROR_TYPE_NONE);
    return 0;
}

#endif /* OVS_UPSTREAM */
#endif /* ALLOW_EXPERIMENTAL_API */

bool is_doca_class(const struct netdev_class *class);
void netdev_doca_dev_lock(struct netdev *netdev);
void netdev_doca_dev_unlock(struct netdev *netdev);

#else

static inline void
free_dpdk_buf(struct dp_packet *buf OVS_UNUSED)
{
    /* Nothing */
}

#endif

#endif /* netdev-doca.h */
